<?php

/**
 * Class clientService
 */
class clientApi
{
    private $curl;
    private $cookie;
    private $cookie_path = "/cookies";
    public $hostname;
    protected $user;
    protected $pass;
    protected $port;
    protected $verify_ssl;
    protected $constructor_success = false;
    public $token = null;

    /**
     * clientApi constructor.
     * @param string $hostname
     * @param string $user
     * @param string $pass
     * @param string int $port
     * @param string bool $verify_ssl
     */
    public function __construct($hostname, $user, $pass, $port = 80, $verify_ssl = false)
    {
        if (empty($hostname) || empty($user) || empty($pass) || empty($port)) {
            $this->constructor_success = false;
            return false;
        }
        // Check port is between 1 and 65535.
        if (!is_int($port) || $port < 1 || $port > 65535) {
            $this->constructor_success = false;
            return false;
        }
        // Check that verify_ssl is boolean.
        if (!is_bool($verify_ssl)) {
            $this->constructor_success = false;
            return false;
        }

        $this->hostname = $hostname;
        $this->user = $user;
        $this->pass = $pass;
        $this->port = $port;
        $this->verify_ssl = $verify_ssl;
        $this->constructor_success = true;

        $this->init();
    }

    /**
     * Initializes the curl object with the default options.
     * If cookie is null it is created
     * @param string $cookie to use for connection
     */
    public function init($cookie = null)
    {
        if ($cookie)
            $this->cookie = $cookie;
        else
            $this->cookie = $this->cookie_path;

        $this->curl = curl_init();
        curl_setopt($this->curl, CURLOPT_HEADER, true);
        curl_setopt($this->curl, CURLOPT_COOKIEFILE, $this->cookie);
        curl_setopt($this->curl, CURLOPT_COOKIEJAR, $this->cookie);
        curl_setopt($this->curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($this->curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($this->curl, CURLOPT_CONNECTTIMEOUT, 5);
        curl_setopt($this->curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($this->curl, CURLOPT_AUTOREFERER, TRUE);
    }

    /**
     * @return bool
     */
    public function login()
    {
        $login_postfields = array();
        $login_postfields['user'] = urlencode($this->user);
        $login_postfields['password'] = urlencode($this->pass);

        $result = $this->get("api/public/login/{$login_postfields['user']}/{$login_postfields['password']}");
        if ($result['success'] == true)
            $this->token = $result['token'];
        else
            return false;
        return true;
    }

    /**
     * Sets the path to which cookies are saved.
     * Important: The apache user must have read and write access
     * @param string $path
     */
    public function setCookiePath($path)
    {
        $this->cookie_path = $path;
    }

    /**
     * Send a GET request to the specified URL
     * @param string $route
     * @param bool $follow
     * @return array Response generated by the server
     */
    public function get($route = null, $follow = false)
    {
        $this->init();
        if (!empty($this->token))
            curl_setopt($this->curl, CURLOPT_HTTPHEADER, array("token: {$this->token}"));
        curl_setopt($this->curl, CURLOPT_URL, "http://{$this->hostname}:{$this->port}/{$route}");
        curl_setopt($this->curl, CURLOPT_POST, false);
        curl_setopt($this->curl, CURLOPT_HEADER, $follow);
        $result = curl_exec($this->curl);
        if ($result === false) {
            echo curl_error($this->curl);
        }
        $this->_close();
        return json_decode($result, true);
    }

    /**
     * Sends a POST request to the specified URL
     * @param string $route
     * @param array $post_elements
     * @param bool $follow
     * @param bool $header
     * @return array Response generated by the server
     */
    public function post($route = null, $post_elements = array(), $follow = false, $header = false)
    {
        $this->init();
        $elements = array();
        if (!empty($this->token))
            curl_setopt($this->curl, CURLOPT_HTTPHEADER, array("token: {$this->token}"));
        $elements = http_build_query($post_elements);
        curl_setopt($this->curl, CURLOPT_URL, "http://{$this->hostname}:{$this->port}/{$route}");
        curl_setopt($this->curl, CURLOPT_POST, true);
        //curl_setopt($this->curl, CURLOPT_REFERER, '');
        curl_setopt($this->curl, CURLOPT_HEADER, $header OR $follow);
        curl_setopt($this->curl, CURLOPT_POSTFIELDS, $elements);
        //curl_setopt($this->curl, CURLOPT_FOLLOWLOCATION, $follow);
        $result = curl_exec($this->curl);
        $this->_close();
        return json_decode($result, true);
    }

    /**
     * @param null $route
     * @param null $post_elements
     * @param bool $follow
     * @return array Response generated by the server
     */
    public function delete($route = null, $follow = false)
    {
        $this->init();
        if (!empty($this->token))
            curl_setopt($this->curl, CURLOPT_HTTPHEADER, array("token: {$this->token}"));
        curl_setopt($this->curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        curl_setopt($this->curl, CURLOPT_URL, "http://{$this->hostname}:{$this->port}/{$route}");
        curl_setopt($this->curl, CURLOPT_POST, false);
        curl_setopt($this->curl, CURLOPT_HEADER, $follow);
        curl_setopt($this->curl, CURLOPT_REFERER, '');
        curl_setopt($this->curl, CURLOPT_FOLLOWLOCATION, $follow);
        $result = curl_exec($this->curl);
        if ($result === false) {
            echo curl_error($this->curl);
        }
        $this->_close();
        return json_decode($result, true);
    }

    /**
     * Download a binary file in the buffer
     * @param string $url
     * @return string
     */
    public function getBinary($url)
    {
        $this->init();
        curl_setopt($this->curl, CURLOPT_URL, "http://{$this->hostname}:{$this->port}");
        curl_setopt($this->curl, CURLOPT_BINARYTRANSFER, 1);
        $result = curl_exec($this->curl);
        $this->_close();
        return $result;
    }

    /**
     * Close the connection
     */
    private function _close()
    {
        curl_close($this->curl);
    }

    public function close()
    {
        if (file_exists($this->cookie))
            unlink($this->cookie);
    }
}